package eu.mhsl.minenet.minigames.instance.room.inventory;

import eu.mhsl.minenet.minigames.instance.game.Game;
import eu.mhsl.minenet.minigames.instance.game.GameList;
import eu.mhsl.minenet.minigames.instance.game.GameType;
import eu.mhsl.minenet.minigames.instance.game.minigame.config.GameConfigurationInventory;
import eu.mhsl.minenet.minigames.instance.game.minigame.config.GameFactory;
import eu.mhsl.minenet.minigames.instance.room.Room;
import eu.mhsl.minenet.minigames.shared.inventory.InteractableInventory;
import eu.mhsl.minenet.minigames.util.InventoryItemAlignment;
import net.kyori.adventure.text.Component;
import net.minestom.server.inventory.InventoryType;
import net.minestom.server.item.ItemHideFlag;
import net.minestom.server.item.ItemStack;
import net.minestom.server.item.Material;

import java.util.Arrays;
import java.util.List;
import java.util.stream.Collectors;

public class MinigameTypeSelectInventory extends InteractableInventory {
    final private Room room;
    public MinigameTypeSelectInventory(Room room) {
        super(InventoryType.CHEST_6_ROW, Component.text("MineNet Servernetzwerk"));
        this.room = room;

        int typeCount = 0;
        InventoryItemAlignment itemAlignment = new InventoryItemAlignment(GameType.values().length, 1);
        for (GameType type : GameType.values()) {
            setClickableItem(
                    ItemStack.builder(type.getIcon())
                            .displayName(type.getTitle().asComponent())
                            .lore(type.getDescription().asComponent())
                            .build(),
                    itemAlignment.next().get(),
                    itemClick -> {
                        drawGames(type);
                    }
            );
            typeCount++;
        }

        for(int i = 9; i <= 17; i++) {
            setDummyItem(Material.CYAN_STAINED_GLASS_PANE, i);
        }


    }

    private void drawGames(GameType type) {
        for(int i = 18; i <= 53; i++) {
            setDummyItem(Material.AIR, i);
        }
        int offset = 18;

        List<GameList> games = Arrays.stream(GameList.values()).filter(gameList -> gameList.getType().equals(type)).toList();

        InventoryItemAlignment itemAlignment = new InventoryItemAlignment(games.size(), 4);
        for (GameList game : games) {
            GameFactory gameFactory = game.getFactory();

            setClickableItem(
                    ItemStack.builder(gameFactory.symbol())
                            .displayName(gameFactory.name())
                            .lore(gameFactory.description())
                            .meta(metaBuilder -> metaBuilder.hideFlag(ItemHideFlag.HIDE_ATTRIBUTES))
                            .build(),
                    offset + itemAlignment.next().get(),
                    itemClick -> itemClick.getPlayer().openInventory(new GameConfigurationInventory(room, gameFactory))
            );
        }
    }
}
